@@ -163,7 +163,7 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visib
             g_free (markup);
             pop_singles (vbox, singles, pw, visible);
             singles = NULL;
-            gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 0);
+            gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, TRUE, 0);
             /* hidden group */
             build_children (vbox, e->children, pw, !e->hidden);
             if (e->hidden)
@@ -201,7 +201,7 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visib
             }
           else
             w = ww;
-          gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 0);
+          gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, TRUE, 0);
           add_tag (e->tag, ww, pw);
         break;
       
@@ -223,10 +223,7 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visib
           datecombo = gtk_date_combo_new ();
           gtk_box_pack_start (GTK_BOX (hbox), datecombo, TRUE, TRUE, 0);
           gtk_date_combo_clear(GTK_DATE_COMBO(datecombo));
-/*          cbsched = gtk_check_button_new_with_label (_("Schedule"));
-          gtk_widget_set_sensitive (cbsched, FALSE);
-          gtk_box_pack_start (GTK_BOX (hbox), cbsched, TRUE, TRUE, 0);
-*/          cbnoyear = gtk_check_button_new_with_label (_("no year"));
+          cbnoyear = gtk_check_button_new_with_label (_("no year"));
           gtk_box_pack_start (GTK_BOX (hbox), cbnoyear, TRUE, TRUE, 0);
           g_signal_connect (G_OBJECT (cbnoyear), "toggled",
 		    G_CALLBACK (on_unknown_year_toggled), datecombo);
@@ -242,7 +239,7 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visib
         case ITEM_IMAGE:
         {
           GtkWidget *l = gtk_label_new (e->name);
-          GtkWidget *hbox, *image, *btn;
+          GtkWidget *hbox, *image, *btn, *sw, *vp;
           pop_singles (vbox, singles, pw, visible);
           singles = NULL;
           
@@ -252,13 +249,20 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visib
 
           btn = gtk_button_new();
           image = gtk_image_new();
-          gtk_container_add(GTK_CONTAINER(btn),image);
-          gtk_box_pack_start (GTK_BOX (hbox), btn, TRUE, TRUE, 0);
+          vp = gtk_viewport_new(NULL, NULL);
+          gtk_viewport_set_shadow_type(GTK_VIEWPORT(vp), GTK_SHADOW_NONE);
+          sw = gtk_scrolled_window_new(NULL, NULL);
+          gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), 
+                                         GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+          gtk_container_add(GTK_CONTAINER(vp), btn);
+          gtk_container_add(GTK_CONTAINER(sw), vp);
+          gtk_container_add(GTK_CONTAINER(btn), image);
+          gtk_box_pack_start (GTK_BOX (hbox), sw, TRUE, TRUE, 0);
           gtk_widget_show_all(hbox);
           gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 
-            gpe_get_boxspacing());
+                              gpe_get_boxspacing());
           g_signal_connect (G_OBJECT (btn), "clicked",
-		    G_CALLBACK (on_edit_bt_image_clicked), image);
+		                    G_CALLBACK (on_edit_bt_image_clicked), image);
           
           add_tag (e->tag, image, pw);
         }
@@ -323,7 +327,7 @@ create_edit (void)
   GtkWidget *notebook2;
   GtkWidget *edit_cancel;
   GtkWidget *edit_save;
-  GtkWidget *edit_delete;
+  GtkWidget *edit_delete = NULL;
   GtkTooltips *tooltips;
   GtkWidget *vbox, *action_area;
 
@@ -353,15 +357,18 @@ create_edit (void)
 
   notebook2 = gtk_notebook_new ();
   gtk_box_pack_start (GTK_BOX (vbox), notebook2, TRUE, TRUE, 0);
-
-  edit_delete = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+  
+  if (mode_landscape || mode_large_screen)
+    edit_delete = gtk_button_new_from_stock (GTK_STOCK_DELETE);
   edit_cancel = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
   edit_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
   GTK_WIDGET_SET_FLAGS (edit_save, GTK_CAN_DEFAULT);
 
-  gtk_widget_set_sensitive (GTK_WIDGET (edit_delete), FALSE);
-
-  gtk_box_pack_start (GTK_BOX (action_area), edit_delete, TRUE, FALSE, 4);
+  if (mode_landscape || mode_large_screen)
+    {
+      gtk_widget_set_sensitive (GTK_WIDGET (edit_delete), FALSE);
+      gtk_box_pack_start (GTK_BOX (action_area), edit_delete, TRUE, FALSE, 4);
+    }
   gtk_box_pack_start (GTK_BOX (action_area), edit_cancel, TRUE, FALSE, 4);
   gtk_box_pack_start (GTK_BOX (action_area), edit_save, TRUE, FALSE, 4);
 
@@ -430,7 +437,7 @@ edit_window (void)
           gtk_misc_set_alignment (GTK_MISC (catlabel), 0.0, 0.5);  
           gtk_box_pack_start (GTK_BOX (cathbox), catbutton, FALSE, FALSE, gpe_get_boxspacing());
           gtk_box_pack_start (GTK_BOX (cathbox), catlabel, TRUE, TRUE, 0);
-          gtk_box_pack_start (GTK_BOX (vbox), cathbox, TRUE, FALSE, gpe_get_border());
+          gtk_box_pack_start (GTK_BOX (vbox), cathbox, FALSE, TRUE, gpe_get_border());
           g_signal_connect (G_OBJECT (catbutton), "clicked",
 		    G_CALLBACK (on_categories_clicked), w);
           g_object_set_data (G_OBJECT (w), "categories-label", catlabel);
@@ -462,7 +469,7 @@ get_categories_list (struct person *p)
   return list;
 }
 
-static gchar *
+gchar *
 build_categories_string (struct person *p)
 {
   gchar *s = NULL;
@@ -573,9 +580,11 @@ edit_person (struct person *p, gboolean isnew)
                 }
               else if (GTK_IS_IMAGE(w))
                 {
-                  g_object_set_data(G_OBJECT(w),"filename",v->value);
-                  if ((v->value) && !access(v->value,R_OK))
-                    gtk_image_set_from_file(GTK_IMAGE(w), v->value);
+                  if ((v->value) && !access(v->value,R_OK)) 
+                    {
+                      g_object_set_data(G_OBJECT(w),"filename",v->value);
+                      gtk_image_set_from_file(GTK_IMAGE(w), v->value);
+                    }
                   else
                     gtk_image_set_from_stock(GTK_IMAGE(w), 
                       GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_BUTTON);
@@ -723,14 +732,14 @@ char *suffix = NULL;
    
   if (!nametext) return;
   if (strlen(nametext) < 3) return;
-      
     sp = g_strdup(nametext);
 
     /* scan first element for title */
     if (sscanf(sp, "%s", e[0]))
       {
-        i = 0;
         struct tag_value *v;
+        i = 0;
+        
         while (titles[i])
           {
             if (!strcasecmp(e[0], titles[i]))
@@ -746,7 +755,7 @@ char *suffix = NULL;
         if (!has_title) /* find user defined title */
           {
             v = db_find_tag(p, "TITLE");
-            if (v && !strcasecmp(e[0], v->value))
+            if (v && v->value && !strcasecmp(e[0], v->value))
               {
                 has_title = TRUE;
                 title = sp;
@@ -792,6 +801,7 @@ char *suffix = NULL;
     switch (num_elements)
       {
         case 0: 
+        case EOF:
         break;
         case 1: 
           last = e[0];
@@ -830,15 +840,18 @@ char *suffix = NULL;
             }
           else
             {
-              first = sp;
-              last = strrchr(sp, ' ') + 1;
-              while (strlen(last) && isblank(last[0]))
-                last++;
-              strrchr(sp, ' ')[0] = 0;
-              middle = strrchr(first, ' ') + 1;
-              while (strlen(middle) && isblank(middle[0]))
-                middle++;
-              strrchr(first, ' ')[0] = 0;
+              if (strlen(g_strstrip(sp)))
+                {
+                  first = sp;
+                  last = strrchr(sp, ' ') + 1;
+                  while (strlen(last) && isblank(last[0]))
+                    last++;
+                  strrchr(sp, ' ')[0] = 0;
+                  middle = strrchr(first, ' ') + 1;
+                  while (strlen(middle) && isblank(middle[0]))
+                    middle++;
+                  strrchr(first, ' ')[0] = 0;
+                }
             }
         break;
       }
@@ -886,7 +899,7 @@ on_edit_save_clicked (GtkButton * button, gpointer user_data)
         }
         else if (GTK_IS_IMAGE(w))
           {
-            text = g_object_get_data(G_OBJECT(w),"filename");
+            text = g_strdup(g_object_get_data(G_OBJECT(w), "filename"));
           }
         else
         {
@@ -934,15 +947,13 @@ store_filename (GtkWidget *widget, gpointer user_data)
 {
   GtkWidget *selector = GTK_WIDGET(user_data);
   GtkWidget *image = g_object_get_data(G_OBJECT(selector), "image");
-  gchar *old_file;
   const gchar *selected_filename =
     gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector));
   
   if (selected_filename)
     {
-       old_file = g_object_get_data(G_OBJECT(image),"filename");
-       g_free(old_file);
-       g_object_set_data(G_OBJECT(image),"filename",g_strdup(selected_filename));
+       g_object_set_data(G_OBJECT(image), "filename", 
+                         g_strdup(selected_filename));
        gtk_image_set_from_file(GTK_IMAGE(image), selected_filename);
     }
 }
@@ -950,11 +961,11 @@ store_filename (GtkWidget *widget, gpointer user_data)
 void
 on_edit_bt_image_clicked (GtkWidget *bimage, gpointer user_data)
 {
-  GtkWidget *filesel = gtk_file_selection_new ("Select image");
+  GtkWidget *filesel = gtk_file_selection_new (_("Select image"));
   
   gtk_window_set_transient_for(GTK_WINDOW(filesel),
                                GTK_WINDOW(gtk_widget_get_toplevel(bimage)));
-  gtk_window_set_modal(GTK_WINDOW(filesel),TRUE);
+  gtk_window_set_modal(GTK_WINDOW(filesel), TRUE);
   gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
   
   g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
@@ -968,7 +979,7 @@ on_edit_bt_image_clicked (GtkWidget *bimage, gpointer user_data)
 		    "clicked", G_CALLBACK (gtk_widget_destroy),
 		    (gpointer) filesel);
   
-  g_object_set_data(G_OBJECT(filesel),"image",user_data);
+  g_object_set_data(G_OBJECT(filesel),"image", user_data);
   
   gtk_widget_show_all (filesel);
 }
@@ -1055,6 +1066,6 @@ on_name_clicked (GtkButton *button, gpointer user_data)
   name = gtk_editable_get_chars(GTK_EDITABLE(e), 0, -1);
   store_name_fields(p, name);
   g_free(name);
-
-  do_edit_name_detail(edit, p);
+  if (do_edit_name_detail(edit, p))
+    update_edit(p, GTK_WIDGET(edit));
 }
