@@ -8,7 +8,7 @@
  */
 
 /*
- * $Id: main.c,v 1.90 2004/09/16 16:57:47 florian Exp $
+ * $Id: main.c,v 1.96 2004/10/09 19:08:36 florian Exp $
  *
  */
 
@@ -21,6 +21,7 @@
 #include <locale.h>
 #include <time.h>
 #include <gdk/gdkkeysyms.h>
+#include <ctype.h>
 
 #include <gpe/init.h>
 #include <gpe/pixmaps.h>
@@ -255,6 +256,172 @@ get_tag_name(const gchar* tag)
   return NULL;
 }
 
+static gboolean
+pop_singles (GtkWidget *vbox, GSList *list, struct person *p)
+{
+  if (list)
+    {
+      guint length = g_slist_length (list);
+      GtkWidget *table = NULL;
+      guint x = 0;
+    
+      while (list)
+        {
+          GSList *next = list->next;
+          edit_thing_t e = list->data;
+          struct tag_value *tv = NULL;
+          GtkWidget *w;
+          GtkWidget *l;
+          gchar *name;
+
+          if (e->tag) 
+             tv = db_find_tag(p, e->tag);
+          if (tv && tv->value)
+            {
+              if (table == NULL) 
+                 table = gtk_table_new (length, 2, FALSE);
+              w = gtk_label_new (NULL);
+              gtk_misc_set_alignment(GTK_MISC(w),0.0,0.5);
+              gtk_label_set_text(GTK_LABEL(w), tv->value);
+              gtk_label_set_line_wrap(GTK_LABEL(w), 
+                                      ((e->type == ITEM_MULTI_LINE) 
+                                       || mode_landscape) ? TRUE : FALSE);
+          
+              if (e->name[0] && e->name[strlen(e->name)-1] != ':')
+                {
+                  name = g_strdup_printf("%s:", e->name);
+                  l = gtk_label_new (name);
+                  g_free(name);
+                }
+              else
+                l = gtk_label_new (e->name);
+              
+              gtk_misc_set_alignment(GTK_MISC(l),0.0,0.0);
+              
+              gtk_table_attach (GTK_TABLE (table),
+                        l,
+                        0, 1, x, x + 1,
+                        GTK_FILL, GTK_FILL, 0, 0);
+              gtk_table_attach (GTK_TABLE (table),
+                        w,
+                        1, 2, x, x + 1,
+                        GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+                        0, 0, 0);
+              x++;
+            }
+          g_slist_free_1 (list);
+          list = next;
+        }
+    if (table)
+      {
+        gtk_table_set_col_spacings (GTK_TABLE (table), gpe_get_boxspacing());
+        gtk_table_set_row_spacings (GTK_TABLE (table), gpe_get_boxspacing());
+        gtk_container_set_border_width (GTK_CONTAINER (table), 0);
+  
+        gtk_widget_show_all(table);
+        gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 4);
+        return TRUE;
+      }
+    }
+  return FALSE;
+}
+
+static gboolean
+build_children (GtkWidget *vbox, GSList *children, struct person *p)
+{
+  gboolean result = FALSE;
+  GSList *child;
+  GSList *singles = NULL;
+
+  for (child = children; child; child = child->next)
+    {
+      edit_thing_t e = child->data;
+      struct tag_value *tv = NULL;
+      GtkWidget *w;
+      
+      if (e && e->tag)
+        {
+          tv = db_find_tag(p, e->tag);
+        }
+      if (!e->hidden)
+        switch (e->type)
+        {
+          case GROUP:
+            {
+              gchar *markup;
+              gboolean res;
+              if (e->children) /* ignore empty groups */
+                {
+                  markup = g_strdup_printf ("<b>%s</b>", e->name);
+                  w = gtk_label_new (NULL);
+                  gtk_label_set_markup (GTK_LABEL (w), markup);
+                  gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
+                  g_free (markup);
+                  result = pop_singles (vbox, singles, p);
+                  singles = NULL;
+                  gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 0);
+                  res = build_children (vbox, e->children, p);
+                  if (!res)
+                    {
+                      gtk_container_remove(GTK_CONTAINER(vbox), w);
+                    }
+                  result |= res;
+                }
+            }
+          break;
+        
+          case ITEM_MULTI_LINE:
+          case ITEM_SINGLE_LINE:
+            singles = g_slist_append (singles, e);
+          break;
+        
+          case ITEM_DATE:
+          {
+            GtkWidget *l;
+            GtkWidget *hbox;
+            result |= pop_singles (vbox, singles, p);
+            singles = NULL;
+            if (tv && tv->value)
+              {
+                int year,month,day;
+                struct tm tm;
+                char buf[32];
+                
+                result = TRUE;
+                l = gtk_label_new (e->name);
+                gtk_misc_set_alignment(GTK_MISC(l), 0.0, 0.5);
+                memset(&tm, 0, sizeof(tm));
+                sscanf(tv->value,"%04d%02d%02d",&year,&month,&day);
+                if (year)
+                  { 
+                    tm.tm_year = year - 1900;
+                    tm.tm_mon = month;
+                    tm.tm_mday = day;
+                    strftime(buf, sizeof(buf), "%x", &tm);
+                  }
+                else 
+                  snprintf(buf,sizeof(buf),"%02d/%02d",month+1,day);
+                w = gtk_label_new(buf);
+                gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
+                hbox = gtk_hbox_new(FALSE, gpe_get_boxspacing());
+                gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, TRUE, 0);
+                gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, TRUE, 0);
+                gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+              }
+          }
+          break;
+          default:
+            /* just ignore */
+          break;
+        }
+    }
+
+  result |= pop_singles (vbox, singles, p);
+  singles = NULL;
+  
+  return result;
+}
+
 static int
 show_details (struct person *p)
 {
@@ -264,13 +431,13 @@ show_details (struct person *p)
   gpointer pchild;
   gchar *tagname;
   struct tag_value *curtag;
-  GSList *iter;
   GList *wlist, *witer;
-  gchar *fieldname, *fielddesc;
+  GSList *page;
+  GtkWidget *label;
+  gchar *text;
 
   table = GTK_TABLE (lookup_widget (mainw, "tabDetail"));
   
-  
   if (mode_landscape) /* we show all available data */
     {
       wlist = gtk_container_get_children (GTK_CONTAINER (table));
@@ -282,92 +449,44 @@ show_details (struct person *p)
           g_list_free (wlist);
         }
       i = 1;
-      if (p != NULL)
-        for (iter = p->data; iter; iter = iter->next)
-          {
-            struct tag_value *id = (struct tag_value *) iter->data;
-            fieldname = get_tag_name(id->tag);
-            /* exception list... */
-            if ((fieldname) 
-                 && strcasecmp(id->tag,"NAME")
-                 && strcasecmp(id->tag,"GIVEN_NAME")
-                 && strcasecmp(id->tag,"MIDDLE_NAME")
-                 && strcasecmp(id->tag,"FAMILY_NAME")
-                 && strcasecmp(id->tag,"HONORIFIC_SUFFIX")
-                 && strcasecmp(id->tag,"TITLE")
-                 && strcasecmp(id->tag,"BIRTHDAY")
-                 && strcasecmp(id->tag,"PHOTO")
-               )
-              {
-                if (fieldname[strlen(fieldname)-1] == ':')
-                  fieldname[strlen(fieldname)-1] = 0;
-                if (strstr(id->tag,"HOME"))
-                  fielddesc = g_strdup_printf("%s (%s):",fieldname,_("home"));
-                else if (strstr(id->tag,"WORK"))
-                  fielddesc = g_strdup_printf("%s (%s):",fieldname,_("work"));
-                else
-                  fielddesc = g_strdup_printf("%s:",fieldname);
-                lleft = gtk_label_new(fielddesc);
-                gtk_misc_set_alignment(GTK_MISC(lleft),0,0);
-                gtk_table_attach(table, lleft, 0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
-                lright = gtk_label_new(id->value);
-                gtk_misc_set_alignment(GTK_MISC(lright),0,0);
-                gtk_table_attach(table, lright, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
-                i++;
-                g_free(fieldname);
-                g_free(fielddesc);
-              }
-            else /* handle special fields */
-              {
-                gchar *ts;
-                if (!strcmp(id->tag,"NAME")) 
-                 {
-                    if (fieldname[strlen(fieldname)-1] == ':')
-                      fieldname[strlen(fieldname)-1] = 0;
-                    lleft = gtk_label_new(NULL);
-                    ts = g_strdup_printf("<b>%s:</b>",_("Name"));
-                    gtk_label_set_markup(GTK_LABEL(lleft),ts);
-                    g_free(ts);
-                    gtk_misc_set_alignment(GTK_MISC(lleft),0,0);
-                    gtk_table_attach(table, lleft, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-                    lright = gtk_label_new(NULL);
-                    ts = g_strdup_printf("<b>%s</b>",id->value);
-                    gtk_label_set_markup(GTK_LABEL(lright),ts);
-                    g_free(ts);
-                    gtk_misc_set_alignment(GTK_MISC(lright),0,0);
-                    gtk_table_attach(table, lright, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-                 }
-                if (!strcmp(id->tag,"BIRTHDAY") && strlen(id->value)>=8) 
-                  {
-                    int year,month,day;
-                    struct tm tm;
-                    char buf[32];
-
-                    memset(&tm, 0, sizeof(tm));
-                    sscanf(id->value,"%04d%02d%02d",&year,&month,&day);
-                    if (year)
-                      { 
-                        tm.tm_year = year - 1900;
-                        tm.tm_mon = month;
-                        tm.tm_mday = day;
-                        strftime(buf, sizeof(buf), "%x", &tm);
-                      }
-                    else 
-                      snprintf(buf,sizeof(buf),"%02d/%02d",month+1,day);
-                    lleft = gtk_label_new(_("Birthday:"));
-                    gtk_misc_set_alignment(GTK_MISC(lleft),0,0);
-                    gtk_table_attach(table, lleft, 0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
-                    lright = gtk_label_new(buf);
-                    gtk_misc_set_alignment(GTK_MISC(lright),0,0);
-                    gtk_table_attach(table, lright, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
-                    i++;
-                  }
-              }                
-          }
+      if (p != NULL) /* add contact data */
+        {
+          GtkWidget *vbox = gtk_vbox_new(FALSE, gpe_get_boxspacing());
+          gchar *catstring = build_categories_string(p);
+        
+          gtk_table_attach(GTK_TABLE(table), vbox, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+          for (page = edit_pages; page; page = page->next) /* we use data layout to structure widgets */
+            {
+               edit_thing_t e = page->data;
+               /* add section content */
+               if (!e->hidden)
+                 build_children (vbox, e->children, p);
+            }
+          if (catstring)
+            {
+              text = g_strdup_printf("%s: %s", _("Categories"), catstring);
+              label = gtk_label_new(NULL);
+              gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+              gtk_label_set_markup(GTK_LABEL(label), text);
+              gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 
+                               GTK_FILL, GTK_FILL, 0, 0);
+              g_free(text);
+            }
+        }
+      else  /* just add some feedback */
+        {
+          text = g_strdup_printf("<b>%s</b>", _("No contact selected."));
+          label = gtk_label_new(NULL);
+          gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+          gtk_label_set_markup(GTK_LABEL(label), text);
+          gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
+                           GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
+          g_free(text);
+        }
       gtk_widget_show_all(GTK_WIDGET(table));
     }
   else /* show configured fields only */
-    {    
+    { 
       for (i = 0; i < table->nrows; i++)
       {
         pchild = g_list_nth_data (table->children, 2 * i);
@@ -377,33 +496,117 @@ show_details (struct person *p)
         gtk_label_set_text (GTK_LABEL (lright), "");
       }
       if (p)
-      {
-        for (i = 0; i < table->nrows; i++)
         {
-          pchild = g_list_nth_data (table->children, 2 * i);
-          if (!pchild)
-            continue;
-          lright = ((GtkTableChild *) pchild)->widget;
-          pchild = g_list_nth_data (table->children, 2 * i + 1);
-          if (!pchild)
-            continue;
-          lleft = ((GtkTableChild *) pchild)->widget;
-          tagname =
-            db_get_config_tag (CONFIG_PANEL,
-                       g_strdup (gtk_label_get_text
-                         (GTK_LABEL (lleft))));
-          curtag = db_find_tag (p, tagname);
-          if (curtag != NULL)
+          for (i = 0; i < table->nrows; i++)
             {
-              gtk_label_set_text (GTK_LABEL (lright), curtag->value);
-            }
-          g_free (tagname);
+              pchild = g_list_nth_data (table->children, 2 * i);
+              if (!pchild)
+                continue;
+              lright = ((GtkTableChild *) pchild)->widget;
+              pchild = g_list_nth_data (table->children, 2 * i + 1);
+              if (!pchild)
+                continue;
+              lleft = ((GtkTableChild *) pchild)->widget;
+              tagname =
+                db_get_config_tag (CONFIG_PANEL,
+                           g_strdup (gtk_label_get_text
+                             (GTK_LABEL (lleft))));
+              curtag = db_find_tag (p, tagname);
+              if (curtag != NULL)
+                {
+                  gtk_label_set_text (GTK_LABEL (lright), curtag->value);
+                }
+              g_free (tagname);
+           }
         }
-      }
     }
   return 0;
 }
 
+static void
+show_details_window(GtkWidget *btn)
+{
+  GtkWidget *dlgDetail;
+  GtkWidget *button, *label;
+  GtkWidget *vbox, *vport, *scrolled_window;
+  gchar *catstring, *text;
+  GtkTreeIter iter;
+  GtkTreeSelection *sel;
+  GtkTreeModel *model;
+  GSList *page;
+  struct person *p;
+  
+  /* get person data */
+  sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
+  if (gtk_tree_selection_get_selected (sel, &model, &iter))
+    {
+      gint id;
+      gtk_tree_model_get (model, &iter, 1, &id, -1);
+      p = db_get_by_uid (id);
+    }
+  else
+    return;
+  
+  /* create and set up dialog */
+  dlgDetail = gtk_dialog_new_with_buttons(_("Contact"), GTK_WINDOW(mainw), 
+                                          GTK_DIALOG_MODAL 
+                                          | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                          NULL);
+  gtk_window_set_default_size(GTK_WINDOW(dlgDetail), 220, 300);
+  button = gtk_dialog_add_button(GTK_DIALOG(dlgDetail), 
+                                 GTK_STOCK_CLOSE, 
+                                 GTK_RESPONSE_OK);
+  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+  gtk_widget_grab_default(button);
+  
+  vbox = gtk_vbox_new(FALSE, gpe_get_boxspacing());
+  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
+                                 GTK_POLICY_AUTOMATIC,
+                                 GTK_POLICY_AUTOMATIC);
+  vport = gtk_viewport_new(NULL, NULL);
+  gtk_viewport_set_shadow_type(GTK_VIEWPORT(vport), GTK_SHADOW_NONE);
+  gtk_container_add(GTK_CONTAINER(vport), vbox);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), vport);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgDetail)->vbox), 
+                     scrolled_window, TRUE, TRUE, 0);
+  
+  /* create content */
+  catstring = build_categories_string(p);
+    
+  for (page = edit_pages; page; page = page->next) /* we use data layout to structure widgets */
+    {
+       edit_thing_t e = page->data;
+
+      if ((!e->hidden) && (e->children))
+        {
+          text = g_strdup_printf ("<b>%s</b>", e->name);
+          label = gtk_label_new (NULL);
+          gtk_label_set_markup (GTK_LABEL (label), text);
+          gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+          g_free (text);
+          gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+          if (!build_children (vbox, e->children, p))
+            gtk_container_remove(GTK_CONTAINER(vbox), label);
+        }
+    }
+  if (catstring)
+    {
+      text = g_strdup_printf("%s: %s", _("Categories"), catstring);
+      label = gtk_label_new(NULL);
+      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+      gtk_label_set_markup(GTK_LABEL(label), text);
+      gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
+      g_free(text);
+    }
+
+  /* show details dialog */
+  gtk_widget_show_all(dlgDetail);
+  gtk_dialog_run(GTK_DIALOG(dlgDetail));
+  gtk_widget_destroy(dlgDetail);
+  discard_person (p);
+}
+
 void
 selection_made (GtkTreeSelection *sel, GObject *o)
 {
@@ -411,12 +614,13 @@ selection_made (GtkTreeSelection *sel, GObject *o)
   guint id;
   struct person *p;
   GtkTreeModel *model;
-  GtkWidget *edit_button, *delete_button, *new_button;
+  GtkWidget *edit_button, *delete_button, *new_button, *details_button;
 
   edit_button = g_object_get_data (o, "edit-button");
   delete_button = g_object_get_data (o, "delete-button");
   new_button = g_object_get_data (o, "new-button");
-
+  details_button = g_object_get_data (o, "details-button");
+  
   if (gtk_tree_selection_get_selected (sel, &model, &iter))
     {
       gtk_tree_model_get (model, &iter, 1, &id, -1);
@@ -429,11 +633,15 @@ selection_made (GtkTreeSelection *sel, GObject *o)
 		
       gtk_widget_set_sensitive (edit_button, TRUE);
       gtk_widget_set_sensitive (delete_button, TRUE);
+      if (!mode_landscape)
+        gtk_widget_set_sensitive (details_button, TRUE);
     }
   else
     {
       gtk_widget_set_sensitive (edit_button, FALSE);
       gtk_widget_set_sensitive (delete_button, FALSE);
+      if (!mode_landscape)
+        gtk_widget_set_sensitive (details_button, FALSE);
     }
 	/* restore after edit/new */
     gtk_widget_set_sensitive (new_button, TRUE);
@@ -448,6 +656,9 @@ match_for_search (struct person *p, const gchar *text, struct gpe_pim_category *
   gchar *name;
   gchar *company;
   
+  if ((text == NULL) && (cat == NULL)) /* some speedup */
+    return TRUE;
+  
   if (p->given_name)
     gn = g_utf8_strdown(p->given_name, -1);
   else
@@ -492,22 +703,6 @@ match_for_search (struct person *p, const gchar *text, struct gpe_pim_category *
   if (name) g_free(name);
   if (company) g_free(company);
     
-/* alternate method if nothing found 
-  if ...
-    
-  else
-    {
-      gchar *lname = g_utf8_strdown (p->name, -1);
-    
-      if (strstr (lname, text) == NULL)
-        {
-          g_free (lname);
-          return FALSE;
-        }
-    }
-
-  g_free (lname);
-*/
   if (cat)
     {
       GSList *l;
@@ -539,8 +734,9 @@ do_search (GObject *obj, GtkWidget *entry)
 {
   gchar *text = g_utf8_strdown (gtk_entry_get_text (GTK_ENTRY (entry)), -1);
   guint category = gtk_option_menu_get_history (GTK_OPTION_MENU (categories_smenu));
-  GSList *all_entries = db_get_entries (), *iter;
+  GSList *all_entries = db_get_entries (), *iter = NULL;
   struct gpe_pim_category *c = NULL;
+  GtkTreePath *path;
 
   if (category)
     {
@@ -561,19 +757,25 @@ do_search (GObject *obj, GtkWidget *entry)
   for (iter = all_entries; iter; iter = iter->next)
     {
       struct person *p = iter->data;
-      GtkTreeIter iter;
+      GtkTreeIter titer;
       
       if (match_for_search (p, text, c))
         {
-	      gtk_list_store_append (list_store, &iter);
-	      gtk_list_store_set (list_store, &iter, 0, p->name, 1, p->id, -1);
+	      gtk_list_store_append (list_store, &titer);
+	      gtk_list_store_set (list_store, &titer, 0, p->name, 1, p->id, -1);
         }
 
       discard_person (p);
     }
   
   g_slist_free (all_entries);
-  g_free (text);
+  if (text) 
+    g_free (text);
+  
+  /* select first */
+  path = gtk_tree_path_new_first();
+  gtk_tree_view_set_cursor(GTK_TREE_VIEW(list_view), path, NULL, FALSE);
+  gtk_tree_path_free(path);
 }
 
 #define SEARCH_DELAY  200
@@ -616,7 +818,8 @@ update_display (void)
 {
   GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
   update_categories ();
-  selection_made(sel, G_OBJECT(gtk_widget_get_toplevel(search_entry)));
+  if (sel)
+    selection_made(sel, G_OBJECT(gtk_widget_get_toplevel(search_entry)));
   do_search (G_OBJECT (search_entry), search_entry);
 }
 
@@ -747,10 +950,27 @@ window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkTreeView *tree)
       return TRUE;
     }
 
+  if ((k->keyval == GDK_Delete) 
+      && (!GTK_WIDGET_HAS_FOCUS(search_entry) 
+          || !strlen(gtk_entry_get_text(GTK_ENTRY(search_entry)))))
+    {
+      delete_contact(widget, NULL);
+      return TRUE;
+    }
+  
   if (k->keyval == GDK_Return)
     {
       edit_contact(NULL, NULL);
-    }  
+    }
+  
+  /* automatic search */
+  if (!k->state && k->string && isalpha(k->string[0]))
+    {
+      if (!GTK_WIDGET_HAS_FOCUS(search_entry))
+        gtk_widget_grab_focus(search_entry);
+      return FALSE;
+    }
+  
   return FALSE;
 }
 
@@ -883,7 +1103,7 @@ create_main (gboolean show_config_button)
   GtkWidget *pDetail;
   GtkWidget *tabDetail;
   GtkWidget *toolbar, *pw;
-  GtkWidget *b, *btnNew;
+  GtkWidget *b, *btnNew, *btnDetails;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkTreeSelection *tree_sel;
@@ -916,7 +1136,7 @@ create_main (gboolean show_config_button)
   gtk_box_pack_start (GTK_BOX (vbox1), toolbar, FALSE, FALSE, 0);
 
   btnNew = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_NEW,
-			    _("New contact"), _("Tap here to add a new contact."),
+			    _("Tap this button to add a new contact."), NULL,
 			    G_CALLBACK (new_contact), NULL, -1);
   g_object_set_data (G_OBJECT (main_window), "new-button", btnNew);
 
@@ -924,17 +1144,27 @@ create_main (gboolean show_config_button)
 							gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar))));
 
   b = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Edit"), 
-			       _("Edit contact"), _("Tap here to edit the selected contact."),
+			       _("Tap here to edit the selected contact."), NULL,
 			       pw, (GtkSignalFunc) edit_contact, NULL);
   g_object_set_data (G_OBJECT (main_window), "edit-button", b);
   gtk_widget_set_sensitive (b, FALSE);
 
   b = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_DELETE,
-				_("Delete contact"), _("Tap here to delete the selected contact."),
+				_("Tap here to delete the selected contact."), NULL,
 				G_CALLBACK (delete_contact), NULL, -1);
   g_object_set_data (G_OBJECT (main_window), "delete-button", b);
   gtk_widget_set_sensitive (b, FALSE);
 
+  if (!mode_landscape)
+    {
+      pw = gtk_image_new_from_stock (GTK_STOCK_INDEX,
+                                     gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
+      btnDetails = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Details"), 
+                                      _("Tap here to show contacts details."), 
+                                      NULL, pw, G_CALLBACK (show_details_window), NULL);
+      g_object_set_data (G_OBJECT (main_window), "details-button", btnDetails);
+    }
+
   if (show_config_button)
     gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_PROPERTIES,
 			    _("Properties"), _("Tap here to configure the program."),
@@ -1048,7 +1278,21 @@ create_main (gboolean show_config_button)
   tabDetail = gtk_table_new (1, 2, FALSE);
   gtk_table_set_col_spacings(GTK_TABLE(tabDetail),gpe_get_boxspacing()*2);
   gtk_container_set_border_width(GTK_CONTAINER(tabDetail),gpe_get_border());
-  gtk_container_add (GTK_CONTAINER (pDetail), tabDetail);
+  if (mode_large_screen)
+    {
+      GtkWidget *vport;
+      scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
+                                     GTK_POLICY_NEVER,
+                                     GTK_POLICY_AUTOMATIC);
+      vport = gtk_viewport_new(NULL, NULL);
+      gtk_viewport_set_shadow_type(GTK_VIEWPORT(vport), GTK_SHADOW_NONE);
+      gtk_container_add(GTK_CONTAINER(vport), tabDetail);
+      gtk_container_add (GTK_CONTAINER (scrolled_window), vport);
+      gtk_container_add (GTK_CONTAINER (pDetail), scrolled_window);
+    }
+  else
+    gtk_container_add (GTK_CONTAINER (pDetail), tabDetail);
   g_object_set_data (G_OBJECT (main_window), "tabDetail", tabDetail);
   g_object_set_data (G_OBJECT (main_window), "entry", entry1);
 
